#/bin/bash 

WORKDIR=/transfer/dbtool

# Fallback
if [ ! -d $WORKDIR ] ; then
   WORKDIR=/tmp/ords$$
   echo "Workdir set to ${WORKDIR}." 
fi


OPTSTRING=":a:o:p:u:v"
APEX_VERSION="latest"
APEX_URL="https://download.oracle.com/otn_software/apex/apex-latest.zip"
ORDS_VERSION="latest"
ORDS_URL="https://download.oracle.com/otn_software/java/ords/ords-latest.zip"
VERBOSE=""
 
JDK_URL='https://download.oracle.com/java/21/latest/jdk-21_linux-x64_bin.tar.gz'
SHA256_URL=${JDK_URL}".sha256"
JDK_FILE="${WORKDIR}/"$(echo ${JDK_URL} | grep -E -o 'jdk-[0-9]{2}_linux-x64_bin.tar.gz')


# If APEX is used, else ignored
APEX_CREDENTIAL_NAME='ApexCredential'
REPO_SERVER_URL='https://artrepo.vegvesen.no/artifactory/'
REPO='libs-snapshot-local'
APEX_CREDENTIAL_ARTIFACT="/no/vegvesen/dbtool/${APEX_CREDENTIAL_NAME}"
APEX_CREDENTIAL_URL="${REPO_SERVER_URL}${REPO}${APEX_CREDENTIAL_ARTIFACT}"

# Artifact that will be uploaded
TARFILE=ords.tar.gz
REPO_URL_TARFILE="${REPO_SERVER_URL}generic-local-snapshots/ikt/dbtool/ords/${TARFILE}"

# Environment in start.sh

ENV_JAVA_HOME=$(tar tzf $JDK_FILE |head -1 | cut -f1 -d'/')

function verbose(){
   if [[ $# -ne 1 ]] ; then 
      echo 'verbose() requires exactly one argument.'
      exit 1
   fi
   if [[ ${VERBOSE}x != x ]] ; then
      echo $1
   fi
}

function download(){
   if [[ $# -ne 1 ]] ; then
      echo "Download function requires url"
      exit 1
   fi
   verbose "Downloading $1"
   curl -f -s -O --output-dir ${WORKDIR} $1
   retcode=$?
   if [[ $retcode -ne 0 ]] ; then
      echo "Download failed with exit code $retcode"
      exit 1
   fi
   verbose "Download of $1 complete."

}

while getopts ${OPTSTRING} opt ; do
   case ${opt} in
      a)
         APEX_VERSION=${OPTARG}
         APEX_URL="https://download.oracle.com/otn_software/apex/apex_${APEX_VERSION}.zip"
         ;;
      o)
         ORDS_VERSION=${OPTARG}
         ORDS_URL="https://download.oracle.com/otn_software/java/ords/ords-${ORDS_VERSION}.zip"
         ;;
      p)
         repo_password=${OPTARG}
         ;;
      u)
         repo_user=${OPTARG}
         ;;
      v)
         VERBOSE=1
         ;;
      :)
         echo "Option -${OPTARG} requires a value"
         exit 1
         ;;
      ?)
         echo "unkown option -${OPTARG}"
         exit 1
         ;;
   esac
done

APEX_FILE=${WORKDIR}/$(echo $APEX_URL | grep -E -o 'apex[-_][a-z0-9.]+.zip')
ORDS_FILE=${WORKDIR}/$(echo $ORDS_URL | grep -E -o 'ords[-_][a-z0-9.]+.zip')

if [ -z $repo_user ] ; then
    read -p "Username for artrepo.vegvesen.no: " repo_user
fi

if [ -z $repo_password ] ; then
    read -s -p Password: repo_password
    echo
fi


# ORDS and APEX do not have separate MD5 or SHA256 checksums for download.
# Checksum for APEX and ORDS are listed on respective download pages:
#  - https://www.oracle.com/tools/downloads/apex-downloads.html
#  - https://www.oracle.com/database/sqldeveloper/technologies/db-actions/download/


rm -f ${JDK_FILE}.sha256

download $SHA256_URL

if [ ! -r $JDK_FILE ] ; then
    download $JDK_URL
elif [  $(sha256sum $JDK_FILE | cut -f1 -d' ') != $(cat ${JDK_FILE}.sha256 )  ] ; then
    verbose "SHA256 of $JDK_FILE does not match, downloading $JDK_URL"
    rm -f $JDK_FILE
    download $JDK_URL
fi

rm -f $ORDS_FILE $APEX_FILE

download $ORDS_URL
download $APEX_URL

rm -rf ${WORKDIR}/apex
unzip -q -d $WORKDIR  $APEX_FILE 'apex/images/*'

rm -rf ${WORKDIR}/{bin,ords.war}

unzip -q -d $WORKDIR $ORDS_FILE bin/ords ords.war

# echo workaround for bug i 23.2 - oppstart feilet på Debian
# Test uten og fjern fra repo 
#cp ords.fix $WORKDIR/bin/ords

# Download ApexCredential - this one updates credential store in APEX with OpenAM credentialID and secret
# Maven artifact location
# Fixme: Add flag for APEX
verbose "Download from repo ${APEX_CREDENTIAL_URL}/maven-metadata.xml" 
version=$(curl -X GET -u "$repo_user:$repo_password" -s ${APEX_CREDENTIAL_URL}/maven-metadata.xml | grep latest | sed "s/.*<latest>\([^<]*\)<\/latest>.*/\1/")
verbose "Download from repo ${APEX_CREDENTIAL_URL}/${version}/maven-metadata.xml"
build=$(curl -X GET -u "$repo_user:$repo_password" -s ${APEX_CREDENTIAL_URL}/${version}/maven-metadata.xml | grep '<value>' | head -1 | sed "s/.*<value>\([^<]*\)<\/value>.*/\1/")
jar="${APEX_CREDENTIAL_NAME}-${build}.jar"
url="${APEX_CREDENTIAL_URL}/${version}/${jar}"

verbose "Download from repo $url"
curl -s -X GET -u "$repo_user:$repo_password" $url -o "${WORKDIR}/ApexCredential.jar"


rm -f provisioning_done

echo Creating tar file
tar czf ${WORKDIR}/${TARFILE} start.sh ordsconfig -C $WORKDIR  bin ords.war ApexCredential.jar $(basename $JDK_FILE) -C apex images 

if [ $? -ne 0 ] ; then
    echo tar command failed. Aborting
    exit 1
fi

echo  upload to Artifactory
curl -X PUT -u "$repo_user:$repo_password" $REPO_URL_TARFILE -T  "${WORKDIR}/$TARFILE"

